==> yahoo-transport-0.8.1-1.4.linux.tar.gz: yahoo-transport-0.8.1-1.4/src/init.c <==
#include <yahootrans.h>

/* handle a packet from jabberd */
result yahoo_phandler(instance i, dpacket p, void *arg)
{
    yahootrans_instance yti = (yahootrans_instance)arg;
    jpacket jp = jpacket_new(p->x);
    yahootrans_jpacket yjp = pmalloco(jp->p, sizeof(_yahootrans_jpacket));
    yahoo_session s;

    yjp->jp = jp;
    yjp->yti = yti;

    if(i == NULL || p == NULL) 
        return r_ERR;

    switch(p->type)
    {
    case p_NONE:
    case p_NORM:
            log_debug("yahoo", "got a packet: %s", xmlnode2str(p->x));
        /* if this packet is for an existing session, use that mtq, else use the general mtq */
        if((s = yahoo_find_session(yti, jp->from)) != NULL && s->state == YAHOO_STATE_ONLINE)
        {
            mtq_send(s->q, p->p, yahoo_parse_jpacket, (void*)yjp);
        }
        else
            mtq_send(yti->q, p->p, yahoo_parse_jpacket, (void*)yjp);

        return r_DONE;
    default:
        log_debug("yahoo", "[YAHOO] ignoring packet from jabberd: %s", xmlnode2str(p->x));
        return r_PASS;
    }
}

void _yahoo_shutdown(xht sessions, const char *key, void *val, void *arg)
{
    yahootrans_instance yti = (yahootrans_instance)arg;
    yahoo_session  s = (yahoo_session)val;
    xmlnode message;

    message = xmlnode_new_tag("message");
    xmlnode_put_attrib(message, "from", s->yahoo_host);
    xmlnode_put_attrib(message, "type", "error");
    xmlnode_put_attrib(message, "to", jid_full(s->user));
    xmlnode_insert_cdata(xmlnode_insert_tag(message, "error"), "Yahoo transport shutting down.", -1);
    xmlnode_put_attrib(xmlnode_get_tag(message, "error"), "code", "502");

    deliver(dpacket_new(message), yti->i);

    yahoo_session_end(s);
}

void yahoo_shutdown(void *arg)
{
    yahootrans_instance yti = (yahootrans_instance)arg;

    log_notice(yti->i->id, "Yahoo-Transport Shutting down Clean...");

    xhash_walk(yti->s, _yahoo_shutdown, (void*)yti);

    ppdb_free(yti->PPDB);
    ppdb_free(yti->y_ppdb);

    xmlnode_free(yti->c);
}

void yahoo_transport(instance i, xmlnode x)
{
    yahootrans_instance yti = pmalloco(i->p, sizeof(_yahootrans_instance));

    log_notice(i->id, "yahoo-transport starting up...");
    yti->i  = i;
    yti->xc = xdb_cache(i);
    yti->q  = mtq_new(i->p);
    yti->c  = xdb_get(yti->xc,jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:yahoo");
    yti->s  = xhash_new(101);
    yti->start_time = pstrdup(i->p, jutil_timestamp());
    

    /* hook the yahoo transport into jabberd */
    register_phandler(i, o_DELIVER, yahoo_phandler, (void*)yti);
    pool_cleanup(i->p, yahoo_shutdown, (void*)yti);
}

==> yahoo-transport-0.8-1.4.linux.tar.gz: yahoo-transport-0.8-1.4/src/init.c <==
#include <yahootrans.h>

/* handle a packet from jabberd */
result yahoo_phandler(instance i, dpacket p, void *arg)
{
    yahootrans_instance yti = (yahootrans_instance)arg;
    jpacket jp = jpacket_new(p->x);
    yahootrans_jpacket yjp = pmalloco(jp->p, sizeof(_yahootrans_jpacket));
    yahoo_session s;

    yjp->jp = jp;
    yjp->yti = yti;

    if(i == NULL || p == NULL) 
        return r_ERR;

    switch(p->type)
    {
    case p_NONE:
    case p_NORM:
            log_debug("yahoo", "got a packet: %s", xmlnode2str(p->x));
        /* if this packet is for an existing session, use that mtq, else use the general mtq */
        if((s = yahoo_find_session(yti, jp->from)) != NULL && s->state == YAHOO_STATE_ONLINE)
        {
            mtq_send(s->q, p->p, yahoo_parse_jpacket, (void*)yjp);
        }
        else
            mtq_send(yti->q, p->p, yahoo_parse_jpacket, (void*)yjp);

        return r_DONE;
    default:
        log_debug("yahoo", "[YAHOO] ignoring packet from jabberd: %s", xmlnode2str(p->x));
        return r_PASS;
    }
}

int _yahoo_shutdown(void *arg, const void *key, void *data)
{
    yahootrans_instance yti = (yahootrans_instance)arg;
    yahoo_session  s = (yahoo_session)data;
    xmlnode message;

    message = xmlnode_new_tag("message");
    xmlnode_put_attrib(message, "from", s->yahoo_host);
    xmlnode_put_attrib(message, "type", "error");
    xmlnode_put_attrib(message, "to", jid_full(s->user));
    xmlnode_insert_cdata(xmlnode_insert_tag(message, "error"), "Yahoo transport shutting down.", -1);
    xmlnode_put_attrib(xmlnode_get_tag(message, "error"), "code", "502");

    deliver(dpacket_new(message), yti->i);

    yahoo_session_end(s);
    return 1;
}

void yahoo_shutdown(void *arg)
{
    yahootrans_instance yti = (yahootrans_instance)arg;

    log_notice(yti->i->id, "Yahoo-Transport Shutting down Clean...");

    ghash_walk(yti->s, _yahoo_shutdown, (void*)yti);

    ppdb_free(yti->PPDB);
    ppdb_free(yti->y_ppdb);

    xmlnode_free(yti->c);
}

void yahoo_transport(instance i, xmlnode x)
{
    yahootrans_instance yti = pmalloco(i->p, sizeof(_yahootrans_instance));

    log_notice(i->id, "yahoo-transport starting up...");
    yti->i  = i;
    yti->xc = xdb_cache(i);
    yti->q  = mtq_new(i->p);
    yti->c  = xdb_get(yti->xc,jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:yahoo");
    yti->s  = (HASHTABLE)ghash_create_pool(i->p, 23, (KEYHASHFUNC)str_hash_code, (KEYCOMPAREFUNC)j_strcmp);
    yti->start_time = pstrdup(i->p, jutil_timestamp());
    

    /* hook the yahoo transport into jabberd */
    register_phandler(i, o_DELIVER, yahoo_phandler, (void*)yti);
    pool_cleanup(i->p, yahoo_shutdown, (void*)yti);
}
